import axios from 'axios';

export namespace Axitor {
    export namespace Url {
        export class Query {
            private res: string = "?";
            constructor(private urlObj: Record<string, any>) { }
            toString(): string {

                Object.keys(this.urlObj).forEach((key, v) => {
                    this.res += key + "=" + this.urlObj[key] + "&";
                })
                return this.res.slice(0, -1);
            }
        }
        export class Route {

            private res: string = "/";

            constructor(private routes: Array<string>) { }

            toString(): string {
                this.routes.forEach((route: any) => {
                    this.res += route + '/';
                });
                return this.res.slice(0, -1);
            }
        }
    }
    export interface Config {
        url: string;
        apiName?: string;
        body?: object | Array<object>;
        headers?: {};
        query?: Url.Query;
        route?: Url.Route;
    }
    export interface Result {
        progress(apiName?: string): void;
        success(apiName?: string, data?: any): void;
        error(apiName?: string, error?: any): void;
    }

    abstract class Activity {
        public abstract launch(config: Config, result?: Result): void;
        urlGen(config: Config) {

            if (config.route) {
                config.url += config.route.toString()
            }
            if (config.query) {
                config.url += config.query.toString();
            }
        }
    }
    export class Get extends Activity {
        public launch(config: Config, result: Result): void {

            this.urlGen(config);

            result.progress(config?.apiName);

            axios.get(config.url, config.headers ? config.headers : {}).then(res => {
                result.success(config.apiName, res.data);
            }).catch(err => {
                result.error(config.apiName, err);
            })
        }
    }
    export class Post extends Activity{
        public launch(config: Config, result: Result): void {
            this.urlGen(config);

            result.progress(config?.apiName);

            axios.post(config.url,config.body, config.headers ? config.headers : {}).then(res => {
                result.success(config.apiName, res.data);
            }).catch(err => {
                result.error(config.apiName, err);
            })
        }
    }
    export class Put extends Activity{
        public launch(config: Config, result: Result): void {
            this.urlGen(config);

            result.progress(config?.apiName);

            axios.put(config.url,config.body, config.headers ? config.headers : {}).then(res => {
                result.success(config.apiName, res.data);
            }).catch(err => {
                result.error(config.apiName, err);
            })
        }
    }
    export class Delete extends Activity {
        public launch(config: Config, result: Result): void {

            this.urlGen(config);

            result.progress(config?.apiName);

            axios.delete(config.url, config.headers ? config.headers : {}).then(res => {
                result.success(config.apiName, res.data);
            }).catch(err => {
                result.error(config.apiName, err);
            })
        }
    }
    export class Method{
        public static Get = new Get();
        public static Put = new Put();
        public static Post = new Post();
        public static Delete = new Delete();
    }
}